# pi-topPULSE

The pi-topPULSE accessory is available to purchase from the [pi-top website](https://pi-top.com/products/accessories).

---
__***Release notes***:__
* Due to the following [Linux kernel issue](https://github.com/raspberrypi/linux/issues/1855), if you are running Linux kernel version 4.9.x previous to 4.9.35, pi-topPULSE may not be fully functional. In particular this issue can prevent the pi-topPULSE LEDs from responding. If you are experiencing these issues, please check your kernel version by typing `uname -r` at the terminal. You can update your kernel version to the latest by running `sudo rpi-update`.

* Currently, pi-topPULSE is only supported on Raspberry Pi 3. This is due to problems setting the UART clock speed on earlier models of the Pi.

---

# Hardware
## Overview
pi-topPULSE is a 7x7 LED array, a speaker and a microphone. Additionally the device features ambient lights which reflect the state of the LED array, 4 around the speaker, and 3 on the underside. pi-topPULSE uses a variety of interfaces to communicate with the Raspberry Pi: the speaker uses I2S, while and the LEDs and microphone use serial (UART) - Tx and Rx respectively. pi-topPULSE can be used either as a HAT or as pi-top addon.

## Device Configuration
pi-topPULSE needs to be initialised in order to be used. When using the device 

### Microphone Sampling Rate
By default, when plugged in, the red LED on the underside will be switched on. This LED indicates the sampling rate of the microphone - the default is 22050Hz, and so if initialised to 22kHz, the red LED being switched off can be used as an indicator that the pi-topPULSE has been initialised. [pi-top's Amazon Alexa Voice Service integration](https://github.com/pi-top/Alexa-Voice-Service-Integration) uses 16kHz, which is denoted with the red LED being on. In this context, the red LED can be considered as an indicator that the pi-topPULSE is capturing audio.

# Software
## pi-topOS
All pi-topPULSE software and libraries are included and configured 'out-of-the-box' as standard on the latest version of pi-topOS (>= 23-06-2017).

Download the latest version of pi-topOS at [https://pi-top.com/get-started](https://pi-top.com/get-started).

### Summary
Everything is already configured to work out-of-the-box! Simply connect a pi-topPULSE to your pi-top.

As previously mentioned, the speaker on the pi-topPULSE uses I2S. This requires some configuration, which will require a reboot from a typical Raspbian configuration using the default sound drivers. This is also true in reverse - if you have configured a pi-topPULSE and you wish to use the standard HDMI or 3.5mm sound outputs, you will require a reboot.

Volume control is handled by the operating system. However, in order for the operating system to detect the soundcard device, another reboot is required after booting into 'I2S mode' for the first time.

### Some more information
This automatic initialisation is done by a software package called `pt-peripheral-cfg`. It contains a program called `pt-peripherals-daemon`, which runs in the background and scans for newly connected devices. If a device is detected, and the appropriate library is installed allow it to be initialised, it will run the approprate command. The `pt-pulse` package on pi-topOS installs and starts this background process, as well as the Python library to allow the pi-topPULSE to be initialised when detected. In the case of pi-topPULSE, it **DOES WHAT?**

## Raspbian
You can add pi-topPULSE functionality by adding the ability to download from the pi-top software repository to your pi-top, and installing the software to work in the same plug-and-play way as on pi-topOS. Simply run the following commands at the terminal:

```
wget http://apt.pi-top.com/add
sudo bash add
sudo apt-get install pt-pulse
```

If you prefer to manually install the packages or want to install a specific set of packages see the [advanced installations page](https://github.com/pi-top/pi-topPULSE/wiki/Advanced-Installation-Methods).

## How it works - 'under the hood'
The library files in this repository make up a Python module which can be used to initialise the pi-topPULSE.

If you wish to initialise the pi-topPULSE manually, `configuration.py` is the module to import, and is designed to be placed inside a folder, which can either be placed in the same location as the file that you want to run, or inside of the appropriate `dist-packages` folder for the version of Python that you are running (**ADD LINK TO INFORMATION ON THIS**). 

Assuming that you have done this correctly, you should be able to initialise the pi-topPULSE with a Python program that looks something like this:

```
from ptpulse import configuration as ptpulsecfg

if ptpulsecfg.reset_device_state(True):
    print("Successfully enabled pi-topPULSE")
else:
    print("Failed to enable pi-topPULSE")
```
**NOTE: all examples assume that the folder that the library is in is named** `ptpulse` **- change as appropriate if required.**

This function will configure the pi-topPULSE to default values, enabling all functionality.

# Using pi-topPULSE:

## Amazon's Alexa Voice Service

See [here](https://github.com/pi-top/Alexa-Voice-Service-Integration) for more on using pi-top's Alexa Voice Service integration.

## Using a custom Python script

Using the `ptpulse` Python module requires root access to function. If you are running a script, make sure that you are running it with root access. You can do this with the "sudo" command:

	sudo python3 my_cool_pulse_script.py


Alternatively, if you are running Python in `IDLE`, please make sure you start LXTerminal and run idle or idle3 with the "sudo" command, like so:

	sudo idle3

# Documentation & Support

**MAKE SURE THAT THESE ARE READY**

* Function reference - http://docs.pimoroni.com/ptpulse/
* GPIO Pinout - http://pinout.xyz/pinout/ptpulse
* Get help - http://support.pi-top.com/support/login/
